<?php

namespace Drupal\search_api_mecab\Plugin\search_api\processor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api_japanese_tokenizer\Plugin\search_api\processor\MaTokenizerBase;
use Drupal\search_api_mecab\MecabManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Performs tokenization of Japanese text for indexing and searching.
 *
 * @SearchApiProcessor(
 *   id = "mecab_tokenizer",
 *   label = @Translation("MeCab tokenizer"),
 *   description = @Translation("Extend the functionality of the Tokenizer to perform tokenization of Japanese text using MeCab for morphological analysis. Since the traditional CJK processing cannot be used concurrently, issues may occur in sentences containing Chinese or Korean text."),
 *   stages = {
 *     "pre_index_save" = 0,
 *     "preprocess_index" = -6,
 *     "preprocess_query" = -6
 *   }
 * )
 */
class MecabTokenizer extends MaTokenizerBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected MecabManagerInterface $mecabManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->mecabManager
      ->setExecutableDir($this->configuration['executable_dir'])
      ->setSystemDictionaryDir($this->configuration['system_dictionary_dir']);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $mecab = $container->get('search_api_mecab.mecab_manager');
    return new static($configuration, $plugin_id, $plugin_definition, $mecab);
  }

  /**
   * {@inheritdoc}
   */
  protected function getPartOfSpeechOptions(): array {
    return [
      '名詞' => $this->t('Noun', [], ['context' => 'Linguistics']),
      '記号' => $this->t('Symbol', [], ['context' => 'Linguistics']),
      '副詞' => $this->t('Adverb', [], ['context' => 'Linguistics']),
      '助詞' => $this->t('Particle', [], ['context' => 'Linguistics']),
      '動詞' => $this->t('Verb', [], ['context' => 'Linguistics']),
      '感動詞' => $this->t('Interjection', [], ['context' => 'Linguistics']),
      '連体詞' => $this->t('Determiner', [], ['context' => 'Linguistics']),
      '形容詞' => $this->t('Adjective', [], ['context' => 'Linguistics']),
      '接頭詞' => $this->t('Prefix', [], ['context' => 'Linguistics']),
      '接続詞' => $this->t('Conjunction', [], ['context' => 'Linguistics']),
      '助動詞' => $this->t('Auxiliary verb', [], ['context' => 'Linguistics']),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    $configuration['excluded_parts_of_speech'] = [
      '記号',
      '助詞',
      '助動詞',
    ];
    $configuration['base_form_convert'] = 1;
    $configuration['executable_dir'] = '';
    $configuration['system_dictionary_dir'] = '';

    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['base_form_convert'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Convert to base form'),
      '#description' => $this->t('When indexing and searching, words are converted into their base form. For example, "言わ", "言い", "言う", "言え", and "言お" are all converted to "言う", which may improve search performance.'),
      '#default_value' => $this->configuration['base_form_convert'],
    ];

    $form['executable_dir'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MeCab executable path'),
      '#description' => $this->t('Specify the full path to the directory containing the MeCab executable. If the web server’s PATH environment variable already includes the mecab command, this field can be left blank. For security reasons, it is not possible to directly specify the name of the executable file.'),
      '#default_value' => $this->configuration['executable_dir'],
      '#field_suffix' => '/mecab',
    ];

    $form['system_dictionary_dir'] = [
      '#type' => 'textfield',
      '#title' => $this->t('MeCab system dictionary path'),
      '#description' => $this->t('Specify the full path to the MeCab dictionary directory. Leave it blank to use the default dictionary.'),
      '#default_value' => $this->configuration['system_dictionary_dir'],
      '#field_suffix' => '/sys.dic',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    $executable_dir = $form_state->getValue('executable_dir');
    if ($executable_dir !== '') {
      $executable_path = $executable_dir . '/mecab';
      if (!file_exists($executable_path)) {
        $form_state->setErrorByName('executable_dir', $this->t('The MeCab executable path is not a valid directory.'));
      }
    }

    $system_dictionary_dir = $form_state->getValue('system_dictionary_dir');
    if ($system_dictionary_dir !== '') {
      $system_dictionary_path = $system_dictionary_dir . '/sys.dic';
      if (!file_exists($system_dictionary_path)) {
        $form_state->setErrorByName('system_dictionary_dir', $this->t('The MeCab system dictionary path is not a valid directory.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function tokenizeText(string $value): array {
    $use_base_form = $this->configuration['base_form_convert'];
    $tokens = [];
    foreach ($this->mecabManager->analyze($value)->getWords() as $word) {
      $surface = $word->getSurface();
      if (self::isJapanese($surface)) {
        if (!$this->isTargetPartOfSpeech($word->getPartOfSpeech())) {
          continue;
        }
        $tokens[] = $use_base_form ? $word->getBaseForm() : $surface;
      }
      elseif ($this->hasMinimumWordLength($surface)) {
        $tokens[] = $surface;
      }
    }
    return $tokens;
  }

}
