<?php

namespace Drupal\search_api_mecab\Service;

use Symfony\Component\Process\Process;

/**
 * MeCab command execution class.
 */
class MecabExecutor {

  /**
   * Path to MeCab command.
   */
  protected string $command = 'mecab';

  /**
   * Sets the directory path for the MeCab binary.
   *
   * @param string $path
   *   The path to the directory where the MeCab binary is located.
   *
   * @return static
   *   Returns the current instance for method chaining.
   */
  public function setExecutableDir(string $path): static {
    $this->command = ($path == '') ? 'mecab' : "{$path}/mecab";
    return $this;
  }

  /**
   * Builds the MeCab command with the given options.
   *
   * @param array $options
   *   An associative array of options to pass to the MeCab command.
   *
   * @return array
   *   An array of the MeCab command and its options.
   */
  protected function buildCommand(array $options): array {
    return array_merge([$this->command], $options);
  }

  /**
   * Executes the MeCab command with the given options.
   *
   * @param array $options
   *   An associative array of options to pass to the MeCab command.
   * @param ?string $input
   *   The input to pass to the MeCab command via standard input.
   *
   * @return string
   *   The trimmed output of the MeCab command, with leading and trailing
   *   whitespace removed.
   */
  public function execute(array $options = [], ?string $input = NULL): string {
    $command = $this->buildCommand($options);
    $process = new Process($command);
    if (isset($input)) {
      $process->setInput(trim($input));
    }
    $process->run();

    $exit_code = $process->getExitCode();
    $is_success = $exit_code === 0;
    if (!$is_success) {
      // The '-D' option will always return exit code 1, so ignore it.
      $has_dictionary_info_option = in_array('-D', $options) || in_array('--dictionary-info', $options);
      if ($exit_code !== 1 || !$has_dictionary_info_option) {
        throw new \RuntimeException(sprintf(
          'Error executing MeCab command: "%s". Exit code: %d. Command: "%s"',
          $process->getErrorOutput(),
          $exit_code,
          implode(' ', $command)
        ));
      }
    }

    return trim($process->getOutput());
  }

  /**
   * Check if mecab command is executable.
   *
   * @return bool
   *   Return true if mecab command is executable.
   */
  public function isExecutable(): bool {
    try {
      $this->execute();
    }
    catch (\RuntimeException $e) {
      return FALSE;
    }
    return TRUE;
  }

}
