<?php

namespace Drupal\search_api_mecab\Service;

use Drupal\search_api_japanese_tokenizer\Service\MaManagerBase;
use Drupal\search_api_mecab\MecabManagerInterface;
use Drupal\search_api_mecab\Utility\MecabParser;

/**
 * MeCab command wrapped class.
 */
class MecabManager extends MaManagerBase implements MecabManagerInterface {

  /**
   * User dictionary directory path.
   */
  protected string $systemDictionaryDir;

  /**
   * Constructor.
   *
   * @param \Drupal\search_api_mecab\Service\MecabExecutor $mecabExecutor
   *   MeCab executor.
   */
  public function __construct(protected MecabExecutor $mecabExecutor) {}

  /**
   * {@inheritdoc}
   */
  public function setExecutableDir(string $path): static {
    $this->mecabExecutor->setExecutableDir($path);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setSystemDictionaryDir(string $path): static {
    $this->systemDictionaryDir = $path;
    return $this;
  }

  /**
   * Retrieves the options for MeCab analysis.
   *
   * @return array
   *   An associative array of options for MeCab analysis.
   */
  protected function buildDictionaryOptions(): array {
    if ($this->systemDictionaryDir != '') {
      return ['-d', $this->systemDictionaryDir];
    }
    return [];
  }

  /**
   * Analysis text by MeCab.
   *
   * @param string $text
   *   The text to analysis.
   *
   * @return static
   *   Return self object.
   */
  public function analyze(string $text): static {
    try {
      $options = $this->buildDictionaryOptions();
      $this->analysisOutput = $this->mecabExecutor->execute($options, $text);
    }
    catch (\Exception $e) {
      throw new \RuntimeException('An error occurred while analyzing text: ' . $e->getMessage());
    }

    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getWords(): array {
    return MecabParser::parseAnalysisOutput($this->analysisOutput);
  }

  /**
   * {@inheritdoc}
   */
  public function getVersion(): string {
    try {
      $output = $this->mecabExecutor->execute(['-v']);
      if (preg_match('/^mecab of (?<version>\d+\.\d+)$/', $output, $matches)) {
        return $matches['version'];
      }
      throw new \UnexpectedValueException('Failed to parse MeCab version: Unexpected output format.');
    }
    catch (\Exception $e) {
      throw new \RuntimeException('An error occurred while retrieving MeCab version: ' . $e->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isExecutable(): bool {
    return $this->mecabExecutor->isExecutable();
  }

}
