<?php

namespace Drupal\search_api_mecab\Utility;

use Drupal\search_api_mecab\Model\MecabWord;
use Drupal\search_api_mecab\MecabWordInterface;

/**
 * Provides parsing MeCab output line.
 */
class MecabParser {

  /**
   * Number of features in MeCab output.
   */
  const MECAB_FEATURE_COUNT = 9;

  /**
   * Feature indexes.
   */
  const FEATURE_PART_OF_SPEECH = 0;
  const FEATURE_PART_OF_SPEECH_DETAIL_1 = 1;
  const FEATURE_PART_OF_SPEECH_DETAIL_2 = 2;
  const FEATURE_PART_OF_SPEECH_DETAIL_3 = 3;
  const FEATURE_INFLECTIONAL_MODEL = 4;
  const FEATURE_INFLECTIONAL_FORM = 5;
  const FEATURE_BASE_FORM = 6;
  const FEATURE_READING_TEXT = 7;
  const FEATURE_PRONUNCIATION = 8;

  /**
   * Parse analysis features.
   *
   * @param string $features_csv
   *   Features CSV string.
   *
   * @return array
   *   Array of features.
   */
  protected static function parseAnalysisFeaturesCsv(string $features_csv): array {
    $features = explode(',', $features_csv);
    if (count($features) < 2) {
      throw new \RuntimeException(sprintf('Insufficient features in line: "%s".', $features_csv));
    }
    return array_pad($features, self::MECAB_FEATURE_COUNT, NULL);
  }

  /**
   * Parse line text of MeCab output.
   *
   * @param string $line
   *   Line text of MeCab output.
   *
   * @return \Drupal\search_api_mecab\MecabWordInterface
   *   Array of MecabWord objects.
   */
  protected static function parseAnalysisLine(string $line): MecabWordInterface {
    // MeCab output line format: `{surface}\t{comma_separated_features}`.
    // Ex: `生まれ\t動詞,自立,*,*,一段,連用形,生まれる,ウマレ,ウマレ`.
    [$surface, $features_csv] = explode("\t", $line);
    $features = self::parseAnalysisFeaturesCsv($features_csv);
    return (new MecabWord())
      ->setSurface($surface)
      ->setPartOfSpeech($features[self::FEATURE_PART_OF_SPEECH])
      ->setBaseForm($features[self::FEATURE_BASE_FORM]);
  }

  /**
   * Parse text of MeCab analysis output.
   *
   * @param string $text
   *   Text of MeCab output.
   *
   * @return \Drupal\search_api_mecab\MecabWordInterface[]
   *   Array of MaWord objects.
   */
  public static function parseAnalysisOutput(string $text): array {
    $lines = explode("\n", $text);
    $lines = array_filter($lines, function ($line) {
      return $line != 'EOS';
    });
    return array_map([self::class, 'parseAnalysisLine'], $lines);
  }

}
