<?php

namespace Drupal\search_api_sudachi\Plugin\search_api\processor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api_japanese_tokenizer\Plugin\search_api\processor\MaTokenizerBase;
use Drupal\search_api_sudachi\SudachiManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Performs tokenization of Japanese text for indexing and searching.
 *
 * @SearchApiProcessor(
 *   id = "sudachi_tokenizer",
 *   label = @Translation("Sudachi tokenizer"),
 *   description = @Translation("Extend the functionality of the Tokenizer to perform tokenization of Japanese text using Sudachi for morphological analysis. Since the traditional CJK processing cannot be used concurrently, issues may occur in sentences containing Chinese or Korean text."),
 *   stages = {
 *     "pre_index_save" = 0,
 *     "preprocess_index" = -6,
 *     "preprocess_query" = -6
 *   }
 * )
 */
class SudachiTokenizer extends MaTokenizerBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    protected SudachiManagerInterface $sudachiManager,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $sudachiManager
      ->setJarPath($this->configuration['jar_path'])
      ->setSettingsFilePath($this->configuration['settings_file_path'])
      ->setSystemDictionaryPath($this->configuration['system_dictionary_path'])
      ->setSplitMode($this->configuration['split_mode']);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $sudachi = $container->get('search_api_sudachi.sudachi_manager');
    return new static($configuration, $plugin_id, $plugin_definition, $sudachi);
  }

  /**
   * {@inheritdoc}
   */
  protected function getPartOfSpeechOptions(): array {
    return [
      '副詞' => $this->t('Adverb', [], ['context' => 'Linguistics']),
      '助詞' => $this->t('Particle', [], ['context' => 'Linguistics']),
      '動詞' => $this->t('Verb', [], ['context' => 'Linguistics']),
      '名詞' => $this->t('Noun', [], ['context' => 'Linguistics']),
      '空白' => $this->t('Whitespace', [], ['context' => 'Linguistics']),
      '記号' => $this->t('Symbol', [], ['context' => 'Linguistics']),
      '代名詞' => $this->t('Pronoun', [], ['context' => 'Linguistics']),
      '助動詞' => $this->t('Auxiliary verb', [], ['context' => 'Linguistics']),
      '形容詞' => $this->t('Adjective', [], ['context' => 'Linguistics']),
      '形状詞' => $this->t('Adjectival noun', [], ['context' => 'Linguistics']),
      '感動詞' => $this->t('Interjection', [], ['context' => 'Linguistics']),
      '接尾辞' => $this->t('Suffix', [], ['context' => 'Linguistics']),
      '接続詞' => $this->t('Conjunction', [], ['context' => 'Linguistics']),
      '接頭辞' => $this->t('Prefix', [], ['context' => 'Linguistics']),
      '連体詞' => $this->t('Determiner', [], ['context' => 'Linguistics']),
      '補助記号' => $this->t('Auxiliary symbol', [], ['context' => 'Linguistics']),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    $configuration['excluded_parts_of_speech'] = [
      '助動詞',
      '助詞',
      '接頭辞',
      '空白',
      '補助記号',
    ];
    $configuration['use_normalized_words'] = 1;
    $configuration['split_mode'] = 'A';
    $configuration['jar_path'] = '';
    $configuration['settings_file_path'] = '';
    $configuration['system_dictionary_path'] = '';

    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['use_normalized_words'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use normalized words'),
      '#description' => $this->t("Sudachi includes a word normalization feature. For example, the word '向日葵' (sunflower) can also be written as 'ひまわり' or 'ヒマワリ,' but it normalizes all variations to '向日葵.' Similarly, verbs like '言う' (to say), '言お,’ '言え,' and '言い' are converted to their base form, '言う'."),
      '#default_value' => $this->configuration['use_normalized_words'],
    ];

    $form['split_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Split mode'),
      '#options' => [
        'A' => $this->t('Unit A'),
        'B' => $this->t('Unit B'),
        'C' => $this->t('Unit C'),
      ],
      '#description' => $this->t('The split mode determines how Sudachi tokenizes text. Unit A refers to short units designed for search purposes, Unit B represents natural units of Japanese similar to dictionary entries, and Unit C corresponds to the exact length of words registered in the Sudachi dictionary. In Japanese, the smallest units are often treated as independent words. Therefore, recognizing morphemes at the smallest unit level can help reduce the likelihood of search omissions.'),
      '#default_value' => $this->configuration['split_mode'],
    ];

    $form['jar_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Sudachi JAR path'),
      '#description' => $this->t('The path to the Sudachi JAR file. For security reasons, the file name must follow the format sudachi.jar or sudachi-x.y.z.jar.'),
      '#default_value' => $this->configuration['jar_path'],
    ];

    $form['settings_file_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Settings file path'),
      '#description' => $this->t('The path to the Sudachi settings file.'),
      '#default_value' => $this->configuration['settings_file_path'],
    ];

    $form['system_dictionary_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('System dictionary path'),
      '#description' => $this->t('The path to the Sudachi dictionary file.'),
      '#default_value' => $this->configuration['system_dictionary_path'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    $jar_path = $form_state->getValue('jar_path');
    if (!empty($jar_path)) {
      if (!preg_match('/\/sudachi(-\d+\.\d+\.\d+)?\.jar$/', $jar_path)) {
        $form_state->setErrorByName('jar_path', $this->t('The Sudachi JAR file name must be in the format sudachi.jar or sudachi-x.y.z.jar.'));
      }
      if (!is_file($jar_path)) {
        $form_state->setErrorByName('jar_path', $this->t('The specified Sudachi executable path is not valid or the sudachi executable is not found.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function tokenizeText(string $value): array {
    $tokens = [];
    foreach ($this->sudachiManager->analyze($value)->getWords() as $word) {
      $surface = $word->getSurface();
      if (self::isJapanese($surface)) {
        if (!$this->isTargetPartOfSpeech($word->getPartOfSpeech())) {
          continue;
        }
        if ($this->configuration['use_normalized_words']) {
          $tokens[] = $word->getNormalizedWord();
        }
        else {
          $tokens[] = $surface;
        }
      }
      elseif ($this->hasMinimumWordLength($surface)) {
        $tokens[] = $surface;
      }
    }
    return $tokens;
  }

}
