<?php

namespace Drupal\search_api_sudachi\Service;

use Symfony\Component\Process\Process;

/**
 * Sudachi command execution class.
 */
class SudachiExecutor {

  /**
   * Path to Sudachi JAR file.
   */
  protected string $jarPath = '';

  /**
   * Sets the path to the Sudachi JAR file.
   *
   * @param string $path
   *   The path to the Sudachi JAR file.
   *
   * @return static
   *   Returns the current instance for method chaining.
   */
  public function setJarPath(string $path): static {
    $this->jarPath = $path;
    return $this;
  }

  /**
   * Executes the Sudachi command with the given options.
   *
   * @param array $options
   *   An associative array of options to pass to the Sudachi command.
   * @param ?string $input
   *   The input to pass to the Sudachi command via standard input.
   *
   * @return string
   *   The trimmed output of the Sudachi command, with leading and trailing
   *   whitespace removed.
   */
  public function execute(array $options = [], ?string $input = NULL): string {
    if ($this->jarPath === '') {
      throw new \RuntimeException('The path to the Sudachi JAR file has not been set.');
    }

    $command = array_merge(['java', '-jar', $this->jarPath], $options);
    $process = new Process($command);

    // If no configuration file or dictionary is specified as an option, the
    // dictionary `system_core.dic` in the current directory will be used by
    // default. The official documentation provides a tutorial for placing the
    // dictionary file in the same location as the JAR file, so the working
    // directory will be set to the directory where the JAR file is located,
    // assuming that scenario.
    $process->setWorkingDirectory(dirname($this->jarPath));

    if (isset($input)) {
      $process->setInput($input);
    }
    $process->mustRun();

    return trim($process->getOutput());
  }

  /**
   * Check if sudachi command is executable.
   *
   * @return bool
   *   Return true if sudachi command is executable.
   */
  public function isExecutable(): bool {
    try {
      $this->execute();
    }
    catch (\Exception $e) {
      return FALSE;
    }
    return TRUE;
  }

}
