<?php

namespace Drupal\search_api_sudachi\Service;

use Drupal\search_api_japanese_tokenizer\Service\MaManagerBase;
use Drupal\search_api_sudachi\SudachiManagerInterface;
use Drupal\search_api_sudachi\Utility\SudachiParser;

/**
 * Sudachi command wrapped class.
 */
class SudachiManager extends MaManagerBase implements SudachiManagerInterface {

  /**
   * Path to Sudachi dictionary file.
   */
  protected string $settingsFilePath = '';

  /**
   * System dictionary path.
   */
  protected string $systemDictionaryPath = '';

  /**
   * Split mode.
   */
  protected string $splitMode = '';

  /**
   * Constructor.
   *
   * @param \Drupal\search_api_sudachi\Service\SudachiExecutor $sudachiExecutor
   *   Sudachi executor.
   */
  public function __construct(protected SudachiExecutor $sudachiExecutor) {}

  /**
   * {@inheritdoc}
   */
  public function setJarPath(string $path): static {
    $this->sudachiExecutor->setJarPath($path);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setSettingsFilePath(string $path): static {
    $this->settingsFilePath = $path;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setSystemDictionaryPath(string $path): static {
    $this->systemDictionaryPath = $path;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setSplitMode(string $mode): static {
    $this->splitMode = $mode;
    return $this;
  }

  /**
   * Builds the Sudachi command with the given options.
   *
   * @return array
   *   An array of the Sudachi command and its options.
   */
  protected function buildAnalysisOptions(): array {
    $options = [];

    if ($this->settingsFilePath != '') {
      $options[] = '-r';
      $options[] = $this->settingsFilePath;
    }
    if ($this->systemDictionaryPath != '') {
      $options[] = '--systemDict';
      $options[] = $this->systemDictionaryPath;

    }
    if ($this->splitMode != '') {
      $options[] = '-m';
      $options[] = $this->splitMode;
    }

    return $options;
  }

  /**
   * Analysis text by Sudachi.
   *
   * @param string $text
   *   The text to analysis.
   *
   * @return static
   *   Return self object.
   */
  public function analyze(string $text): static {
    try {
      $options = $this->buildAnalysisOptions();
      $this->analysisOutput = $this->sudachiExecutor->execute($options, $text);
    }
    catch (\Exception $e) {
      throw new \RuntimeException('An error occurred while analyzing text: ' . $e->getMessage());
    }

    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isExecutable(): bool {
    return $this->sudachiExecutor->isExecutable();
  }

  /**
   * {@inheritdoc}
   */
  public function getWords(): array {
    return SudachiParser::parseAnalysisOutput($this->analysisOutput);
  }

}
