<?php

namespace Drupal\search_api_japanese_tokenizer\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api_japanese_tokenizer\MaManagerInterface;

/**
 * Provides a base form for testing Japanese morphological analysis.
 */
abstract class MaTestFormBase extends JapaneseTokenizerTestFormBase {

  /**
   * The prefix used for Search API index configuration IDs.
   */
  const SEARCH_API_INDEX_PREFIX = 'search_api.index.';

  /**
   * The constructor.
   *
   * @param \Drupal\search_api_japanese_tokenizer\MaManagerInterface $maManager
   *   The Sudachi service.
   */
  public function __construct(protected MaManagerInterface $maManager) {}

  /**
   * Gets the processor ID defined in the tokenizer plugin annotation.
   *
   * @return string
   *   The unique ID of the tokenizer processor.
   */
  abstract protected function getProcessorId(): string;

  /**
   * Retrieves a list of Search API indexes where this processor is enabled.
   *
   * @return array
   *   An associative array of available Search API indexes, keyed by
   *   configuration name.
   */
  protected function getSearchApiIndexConfigList(): array {
    $config_factory = $this->configFactory();
    $processor_id = $this->getProcessorId();
    $options = [];
    foreach ($config_factory->listAll() as $config_name) {
      if (strpos($config_name, self::SEARCH_API_INDEX_PREFIX) !== 0) {
        continue;
      }

      $config = $config_factory->get($config_name);
      $processor_settings = $config->get('processor_settings');
      if (isset($processor_settings[$processor_id])) {
        $options[$config_name] = $config->get('name');
      }
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $options = $this->getSearchApiIndexConfigList();
    $form['search_api_index_config'] = [
      '#type' => 'select',
      '#title' => $this->t('Search API index'),
      '#description' => $this->t('Select the Search API index to use for testing. Note that this processor must be enabled in the index settings. Part-of-speech filters and other specific settings will not be applied in this test form.'),
      '#options' => $options,
      '#default_value' => reset($options) ?: NULL,
      '#required' => TRUE,
    ];

    $form = array_merge($form, parent::buildForm($form, $form_state));

    $ma_raw_output = $form_state->get('ma_raw_output');
    $form['ma_raw_output'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Morphological analysis raw output'),
      '#default_value' => $ma_raw_output,
      '#disabled' => TRUE,
    ];

    return $form;
  }

  /**
   * Retrieves the processor settings for the specified Search API index.
   *
   * @param string $config_name
   *   The name of the Search API index configuration.
   *
   * @return array
   *   An associative array of processor settings for the given index.
   */
  protected function getSearchApiProcessorSettings(string $config_name): array {
    $config = $this->configFactory()->get($config_name);
    return $config->get('processor_settings.' . $this->getProcessorId());
  }

  /**
   * Configure the Japanese morphological analysis manager.
   *
   * @param string $config_name
   *   The name of the Search API index configuration.
   */
  abstract protected function configureMaManager(string $config_name);

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->setRebuild(TRUE);

    $config_name = $form_state->getValue('search_api_index_config');
    $this->configureMaManager($config_name);
    try {
      $start_time = microtime(TRUE);
      $this->maManager->analyze($form_state->getValue('input_text'));
      $end_time = microtime(TRUE);

      $words = $this->maManager->getWords();
      $tokens = [];
      foreach ($words as $word) {
        $tokens[] = $word->getSurface();
      }
      $tokenized_text = implode(self::WORD_DELIMITER, $tokens);
      $form_state->set('tokenized_text', $tokenized_text);
      $form_state->set('ma_raw_output', $this->maManager->getRawOutput());
      $execution_time = ($end_time - $start_time);
      $this->messenger()->addStatus($this->t('Morphological analysis completed successfully in @time seconds.', ['@time' => number_format($execution_time, 2)]));
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('An error occurred during morphological analysis: @message', ['@message' => $e->getMessage()]));
    }
  }

}
