<?php

namespace Drupal\search_api_japanese_tokenizer\Plugin\search_api\processor;

use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api\Plugin\search_api\processor\Tokenizer;
use Drupal\search_api\SearchApiException;
use Drupal\search_api\Utility\Utility;

/**
 * Abstract base class for tokenizing Japanese text into individual words.
 *
 * Provides functionality to split Japanese text into tokens for indexing
 * and searching in Search API.
 */
abstract class JapaneseTokenizerBase extends Tokenizer {

  /**
   * The regular expression class for Japanese characters.
   */
  const PREG_JAPANESE_CLASS = '\p{Hiragana}\p{Katakana}\p{Han}！-～';

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    $configuration['overlap_cjk'] = 0;
    $configuration['minimum_word_length'] = 3;

    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // The tokenizer’s CJK processing is no longer accessible in this context.
    // This setting is required by the prepare() method and cannot be removed.
    $form['overlap_cjk']['#access'] = FALSE;

    // Disable the parent class minimum word size setting.
    unset($form['minimum_word_size']);

    return $form;
  }

  /**
   * Determines if a word meets the minimum length requirement.
   *
   * @param string $word
   *   The word to check.
   *
   * @return bool
   *   TRUE if the word meets or exceeds the minimum length, FALSE otherwise.
   */
  protected function hasMinimumWordLength(string $word): bool {
    return mb_strlen($word) >= $this->configuration['minimum_word_length'];
  }

  /**
   * Tokenizes a Japanese text string into an array of tokens.
   *
   * This method must be implemented by concrete classes to perform
   * language-specific tokenization.
   *
   * @param string $value
   *   The Japanese text to tokenize.
   *
   * @return array
   *   An array of tokenized strings.
   */
  abstract protected function tokenizeText(string $value): array;

  /**
   * {@inheritdoc}
   */
  protected function processFieldValue(&$value, $type) {
    $this->prepare();

    $text = $this->simplifyText($value);
    $tokens = $this->tokenizeText($text);
    $value = array_map([Utility::class, 'createTextToken'], $tokens);
  }

  /**
   * {@inheritdoc}
   */
  protected function process(&$value) {
    $this->prepare();

    $text = $this->simplifyText($value);
    $tokens = $this->tokenizeText($text);
    $text = implode(' ', $tokens);

    if ($text === '') {
      throw new SearchApiException('The entered keyword cannot be searched.');
    }
    $value = $text;
  }

}
