<?php

namespace Drupal\search_api_japanese_tokenizer\Plugin\search_api\processor;

use Drupal\Core\Form\FormStateInterface;

/**
 * Splits text into individual words for searching.
 *
 * This tokenizer uses morphological analysis to split Japanese text into
 * meaningful words, ensuring accurate and effective search indexing.
 */
abstract class MaTokenizerBase extends JapaneseTokenizerBase {

  /**
   * Part of speech filters.
   */
  protected array $excludedPartsOfSpeech;

  /**
   * Returns the part of speech options.
   *
   * @return array
   *   The part of speech options.
   */
  abstract protected function getPartOfSpeechOptions(): array;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    $configuration['excluded_parts_of_speech'] = [];

    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['minimum_word_length'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum number of characters for non-Japanese words to be indexed'),
      '#default_value' => $this->configuration['minimum_word_length'],
      '#min' => 1,
      '#max' => 1000,
      '#field_suffix' => $this->t('characters', [], ['context' => 'Length of a word']),
      '#description' => $this->t('The minimum number of characters required to index non-Japanese words. A lower setting means better search result ranking, but also a larger database. Each search query must contain at least one keyword that is this size (or longer).'),
    ];

    $form['excluded_parts_of_speech'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Excluded parts of speech'),
      '#options' => $this->getPartOfSpeechOptions(),
      '#description' => $this->t('The selected parts of speech here will be excluded from the search index.'),
      '#default_value' => $this->configuration['excluded_parts_of_speech'],
    ];

    return $form;
  }

  /**
   * Prepares the processor by setting properties.
   */
  protected function prepare() {
    parent::prepare();

    if (!isset($this->excludedPartsOfSpeech)) {
      $this->excludedPartsOfSpeech = array_filter($this->configuration['excluded_parts_of_speech']);
    }
  }

  /**
   * Check if the given part of speech is part of the filter.
   *
   * @return bool
   *   TRUE if the part of speech is part of the filter, FALSE otherwise.
   */
  protected function isTargetPartOfSpeech($part_of_speech): bool {
    // If the filter is empty, all parts of speech are included.
    if (empty($this->excludedPartsOfSpeech)) {
      return TRUE;
    }
    if (in_array($part_of_speech, $this->excludedPartsOfSpeech)) {
      return FALSE;
    }
    return TRUE;
  }

  /**
   * Check if the given string is Japanese.
   *
   * @param string $string
   *   The string to check.
   *
   * @return bool
   *   TRUE if the string contains Japanese characters, FALSE otherwise.
   */
  protected static function isJapanese($string): bool {
    return preg_match('/[' . self::PREG_JAPANESE_CLASS . ']/u', $string);
  }

}
