<?php

namespace Drupal\search_api_japanese_tokenizer\Plugin\search_api\processor;

use Drupal\Core\Form\FormStateInterface;

/**
 * Splits text into words to improve search with machine learning.
 *
 * This tokenizer uses machine learning to segment Japanese text into meaningful
 * units.
 */
abstract class MlTokenizerBase extends JapaneseTokenizerBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();
    $configuration['minimum_hiragana_word_length'] = 2;
    $configuration['minimum_katakana_word_length'] = 2;
    $configuration['minimum_kanji_word_length'] = 1;

    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $parents = $form['#parents'];

    $form['minimum_word_length'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Minimum word length to index'),
      '#description' => $this->t('If a word matches any of the specified conditions here, it will be included in the search index. A lower setting means better search result ranking, but also a larger database. Each search query must contain at least one keyword that is this size (or longer).'),
    ];

    $form['minimum_word_length']['minimum_hiragana_word_length'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum number of characters for words containing Hiragana'),
      '#default_value' => $this->configuration['minimum_hiragana_word_length'],
      '#min' => 1,
      '#max' => 1000,
      '#field_suffix' => $this->t('characters', [], ['context' => 'Length of a word']),
      '#description' => $this->t('The minimum number of Hiragana characters a word must contain to be indexed. Since Hiragana words are often used for conjunctions and other small elements, setting a low threshold may affect search accuracy.'),
      '#parents' => array_merge($parents, ['minimum_hiragana_word_length']),
    ];

    $form['minimum_word_length']['minimum_katakana_word_length'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum number of characters for words containing Katakana'),
      '#default_value' => $this->configuration['minimum_katakana_word_length'],
      '#min' => 1,
      '#max' => 1000,
      '#field_suffix' => $this->t('characters', [], ['context' => 'Length of a word']),
      '#description' => $this->t('The minimum number of Katakana characters a word must contain to be indexed. Single-character Katakana words are uncommon, so there is no need to set a low threshold.'),
      '#parents' => array_merge($parents, ['minimum_katakana_word_length']),
    ];

    $form['minimum_word_length']['minimum_kanji_word_length'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum number of characters for words containing Kanji'),
      '#default_value' => $this->configuration['minimum_kanji_word_length'],
      '#min' => 1,
      '#max' => 1000,
      '#field_suffix' => $this->t('characters', [], ['context' => 'Length of a word']),
      '#description' => $this->t('The minimum number of Kanji characters a word must contain to be indexed. In Japanese, a single Kanji character often carries significant meaning, so setting this to 1 is recommended.'),
      '#parents' => array_merge($parents, ['minimum_kanji_word_length']),
    ];

    $form['minimum_word_length']['minimum_word_length'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum number of characters for any words'),
      '#default_value' => $this->configuration['minimum_word_length'],
      '#min' => 1,
      '#max' => 1000,
      '#field_suffix' => $this->t('characters', [], ['context' => 'Length of a word']),
      '#description' => $this->t('The minimum number of characters to index regardless of character type.'),
      '#parents' => array_merge($parents, ['minimum_word_length']),
    ];

    return $form;
  }

  /**
   * Validate whether the given input contains Hiragana characters.
   *
   * Ex: ひらがな.
   *
   * @param string $word
   *   The word to check.
   *
   * @return bool
   *   Returns TRUE if the given word contains Hiragana; otherwise, FALSE.
   */
  protected function hasHiragana(string $word): bool {
    return preg_match('/\p{Hiragana}/u', $word);
  }

  /**
   * Validate whether the given input contains Katakana characters.
   *
   * Ex: カタカナ.
   *
   * @param string $word
   *   The word to check.
   *
   * @return bool
   *   Returns TRUE if the given word contains Katakana; otherwise, FALSE.
   */
  protected function hasKatakana(string $word): bool {
    return preg_match('/\p{Katakana}/u', $word);
  }

  /**
   * Validate whether the given input contains Kanji characters.
   *
   * Ex: 漢字.
   *
   * @param string $word
   *   The word to check.
   *
   * @return bool
   *   Returns TRUE if the given word contains Kanji; otherwise, FALSE.
   */
  protected function hasKanji(string $word): bool {
    return preg_match('/\p{Han}/u', $word);
  }

  /**
   * Check if the given value is Hiragana and meets the specified min length.
   *
   * @param string $word
   *   The word to check.
   *
   * @return bool
   *   TRUE if the word is Hiragana and meets the minimum length, FALSE
   *   otherwise.
   */
  protected function isHiraganaWithMinLength(string $word): bool {
    return $this->hasHiragana($word)
      && mb_strlen($word) >= $this->configuration['minimum_hiragana_word_length'];
  }

  /**
   * Check if the given value is Katakana and meets the specified min length.
   *
   * @param string $word
   *   The word to check.
   *
   * @return bool
   *   TRUE if the word is Katakana and meets the minimum length, FALSE
   *   otherwise.
   */
  protected function isKatakanaWithMinLength(string $word): bool {
    return $this->hasKatakana($word)
      && mb_strlen($word) >= $this->configuration['minimum_katakana_word_length'];
  }

  /**
   * Check if the given value is Kanji and meets the specified min length.
   *
   * @param string $word
   *   The word to check.
   *
   * @return bool
   *   TRUE if the word is Kanji and meets the minimum length, FALSE otherwise.
   */
  protected function isKanjiWithMinLength(string $word): bool {
    return $this->hasKanji($word)
      && mb_strlen($word) >= $this->configuration['minimum_kanji_word_length'];
  }

  /**
   * Validate if the given word matches the conditions for indexing.
   *
   * @param string $word
   *   The word to check.
   *
   * @return bool
   *   TRUE if the word meets the conditions for indexing, FALSE otherwise.
   */
  protected function isIndexableWord(string $word): bool {
    return $this->configuration['minimum_word_length'] == 1
      || is_numeric($word)
      || $this->hasMinimumWordLength($word)
      || $this->isHiraganaWithMinLength($word)
      || $this->isKatakanaWithMinLength($word)
      || $this->isKanjiWithMinLength($word);
  }

}
